from kitconcept.seo import PACKAGE_NAME

import pytest


class TestSetupUninstall:
    @pytest.fixture(autouse=True)
    def uninstalled(self, installer):
        installer.uninstall_product(PACKAGE_NAME)

    def test_addon_uninstalled(self, installer):
        """Test if kitconcept.seo is uninstalled."""
        assert installer.is_product_installed(PACKAGE_NAME) is False

    def test_browserlayer_not_registered(self, browser_layers):
        """Test that IKitconceptSeoLayer is not registered."""
        from kitconcept.seo.interfaces import IKitconceptSeoLayer

        assert IKitconceptSeoLayer not in browser_layers



from kitconcept.seo.testing import FUNCTIONAL_TESTING
from kitconcept.seo.testing import INTEGRATION_TESTING
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.restapi.testing import RelativeSession
from pytest_plone import fixtures_factory

import pytest


pytest_plugins = ["pytest_plone"]


globals().update(
    fixtures_factory(
        (
            (FUNCTIONAL_TESTING, "functional"),
            (INTEGRATION_TESTING, "integration"),
        )
    )
)


@pytest.fixture
def manager_plone_client(functional):
    portal = functional["portal"]
    api_session = RelativeSession(f"{portal.absolute_url()}")
    api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
    return api_session



from kitconcept.seo import PACKAGE_NAME


class TestSetupInstall:
    def test_addon_installed(self, installer):
        """Test if kitconcept.seo is installed."""
        assert installer.is_product_installed(PACKAGE_NAME) is True

    def test_browserlayer(self, browser_layers):
        """Test that IKitconceptSeoLayer is registered."""
        from kitconcept.seo.interfaces import IKitconceptSeoLayer

        assert IKitconceptSeoLayer in browser_layers

    def test_latest_version(self, profile_last_version):
        """Test latest version of default profile."""
        assert profile_last_version(f"{PACKAGE_NAME}:default") == "1000"



from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.testing.zope import WSGI_SERVER_FIXTURE

import kitconcept.seo  # noQA


class Layer(PloneSandboxLayer):
    defaultBases = (PLONE_APP_CONTENTTYPES_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        import plone.restapi

        self.loadZCML(package=kitconcept.seo)
        self.loadZCML(package=plone.restapi)

    def setUpPloneSite(self, portal):
        applyProfile(portal, "kitconcept.seo:default")
        applyProfile(portal, "plone.restapi:default")


FIXTURE = Layer()

INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,),
    name="Kitconcept.SeoLayer:IntegrationTesting",
)


FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE, WSGI_SERVER_FIXTURE),
    name="Kitconcept.SeoLayer:FunctionalTesting",
)



from kitconcept.seo.behaviors.seo import ISeo
from plone.dexterity.interfaces import IDexterityContent
from zope.component import adapter
from ZPublisher.interfaces import IPubAfterTraversal
from ZPublisher.pubevents import PubAfterTraversal


@adapter(IPubAfterTraversal)
def add_x_robots_noindex_header(event: PubAfterTraversal):
    """Add X-Robots-Tag to prevent indexing items with seo_noindex."""
    request = event.request
    for obj in request["PARENTS"]:
        if IDexterityContent.providedBy(obj):
            if ISeo.providedBy(obj) and obj.seo_noindex:
                request.response.setHeader("X-Robots-Tag", "noindex")
            break



from kitconcept.seo import _
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.namedfile.field import NamedBlobImage
from plone.supermodel import model
from zope import schema
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


@provider(IFormFieldProvider)
class ISeo(model.Schema):
    model.fieldset(
        "seo",
        label=_("SEO"),
        fields=[
            "seo_title",
            "seo_description",
            "seo_noindex",
            "seo_canonical_url",
            #            "seo_nofollow",
            #            "seo_noarchive",
            #            "seo_nosnippet",
            "opengraph_title",
            "opengraph_description",
            "opengraph_image",
        ],
    )

    seo_title = schema.TextLine(
        title=_("Title"),
        description=_(
            "Override the meta title. When empty the default title will "
            + "be used. Use maximum 50 characters."
        ),
        required=False,
    )

    seo_description = schema.TextLine(
        title=_("Description"),
        description=_(
            "Override the meta description. When empty the default "
            + "description will be used. Use maximum 150 characters."
        ),
        required=False,
    )

    # https://support.google.com/webmasters/answer/93710?hl=en
    seo_noindex = schema.Bool(
        title=_("No Index"),
        description=_("Prevents a page from appearing in search engines"),
        required=False,
    )

    # https://support.google.com/webmasters/answer/139066?hl=en
    seo_canonical_url = schema.URI(
        title=_("Canonical URL"),
        description=_(
            "Tells the search engine to choose this URL as the canonical version and crawl that."
        ),
        required=False,
    )

    # # https://support.google.com/webmasters/answer/96569?hl=en
    # seo_nofollow = schema.Bool(
    #     title=_(u"No Follow"),
    #     description=_(u"Prevents search engines to follow links on this page"),
    #     required=False,
    # )

    # # https://support.google.com/webmasters/answer/79812?hl=en
    # seo_noarchive = schema.Bool(
    #     title=_(u"No Archive"),
    #     description=_(
    #         u"Prevents search engines to store a cached copy of this page"),
    #     required=False,
    # )

    # # https://support.google.com/webmasters/answer/96569?hl=en
    # seo_nosnippet = schema.Bool(
    #     title=_(u"No Snippet"),
    #     description=_(
    #         u"Prevents search engines from displaying a snippet for your page in search results"  # noqa
    #     ),
    #     required=False,
    # )

    opengraph_title = schema.TextLine(
        title=_("Open Graph Title"),
        description=_(
            "Override the Open Graph title, that Facebook and Twitter use. When empty the default title will "
            + "be used. Use maximum 60 characters."
        ),
        required=False,
    )

    opengraph_description = schema.TextLine(
        title=_("Open Graph Description"),
        description=_(
            "Override the Open Graph description, that Facebook and Twitter use. When empty the default "
            + "description will be used. Use maximum 155 characters."
        ),
        required=False,
    )

    opengraph_image = NamedBlobImage(
        title=_("Open Graph Image"),
        description=_(
            "Override the Open Graph image, that Facebook and Twitter use. When empty the default "
            + "lead image will be used. Recommended image ratio is 1,91:1 and 1200 x 630px."
        ),
        required=False,
    )


@implementer(ISeo)
@adapter(IDexterityContent)
class Seo:
    def __init__(self, context):
        self.context = context



def test_seo_behavior_fields(manager_plone_client):
    # Enable behavior for pages
    manager_plone_client.patch(
        "++api++/@controlpanels/dexterity-types/Document", json={"kitconcept.seo": True}
    )

    # Check schema
    schema = manager_plone_client.get("++api++/@types/Document").json()
    assert schema["fieldsets"][-1] == {
        "behavior": "plone",
        "description": "",
        "fields": [
            "seo_title",
            "seo_description",
            "seo_noindex",
            "seo_canonical_url",
            "opengraph_title",
            "opengraph_description",
            "opengraph_image",
        ],
        "id": "seo",
        "title": "SEO",
    }


def test_noindex_sets_response_header(manager_plone_client):
    # Enable behavior for pages
    manager_plone_client.patch(
        "++api++/@controlpanels/dexterity-types/Document", json={"kitconcept.seo": True}
    )

    # Add page
    resp = manager_plone_client.post(
        "++api++/",
        json={"@type": "Document", "title": "Test page", "seo_noindex": True},
    )
    assert resp.status_code == 201

    # Confirm the page is served with X-Robots-Tag header
    resp = manager_plone_client.get("/test-page")
    assert resp.status_code == 200
    assert resp.headers["X-Robots-Tag"] == "noindex"

    # Confirm views of the page are served with X-Robots-Tag header
    resp = manager_plone_client.get("/test-page/@@view")
    assert resp.status_code == 200
    assert resp.headers["X-Robots-Tag"] == "noindex"



"""Installer for the kitconcept.seo package."""

from pathlib import Path
from setuptools import find_namespace_packages
from setuptools import setup


long_description = f"""
{Path("README.md").read_text()}\n
{Path("CONTRIBUTORS.md").read_text()}\n
{Path("CHANGES.md").read_text()}\n
"""


setup(
    name="kitconcept.seo",
    version="2.1.1",
    description="SEO optimizations plugin for Plone",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 6.0",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Operating System :: OS Independent",
    ],
    keywords="Python Plone CMS",
    author="kitconcept GmbH",
    author_email="info@kitconcept.com",
    url="https://pypi.python.org/pypi/kitconcept.seo",
    project_urls={
        "PyPI": "https://pypi.python.org/pypi/kitconcept.seo",
        "Source": "https://github.com/kitconcept/kitconcept.seo",
        "Tracker": "https://github.com/kitconcept/kitconcept.seo/issues",
    },
    license="GPL version 2",
    packages=find_namespace_packages(where="src/", include=["kitconcept.*"]),
    namespace_packages=["kitconcept"],
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.8",
    install_requires=[
        "setuptools",
        "Products.CMFPlone",
        "plone.app.dexterity",
        "plone.autoform",
        "plone.behavior",
        "plone.dexterity",
        "plone.namedfile",
        "plone.supermodel",
    ],
    extras_require={
        "test": [
            "zest.releaser[recommended]",
            "zestreleaser.towncrier",
            "plone.app.contenttypes[test]",
            "plone.app.testing",
            "plone.restapi[test]",
            # Undeclared dependency of plone.restapi,
            # can be removed after next release
            "plone.app.iterate",
            "pytest",
            "pytest-cov",
            "pytest-plone>=0.2.0",
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    [console_scripts]
    update_dist_locale = kitconcept.seo.locales.update:update_locale
    """,
)
